@@ -3,6 +3,7 @@
 // SPDX-License-Identifier: LGPL-3.0-or-later
 
 #include "variantValue.h"
+#include "constant.h"
 #include <sstream>
 
 std::unique_ptr<VariantValue> creatValueHandler(msg_ptr &msg, DBusValueType type)
@@ -34,29 +35,21 @@ int StringValue::appendValue(std::string &&value) noexcept
 
 int ASValue::openVariant() noexcept
 {
-    return sd_bus_message_open_container(msgRef(), SD_BUS_TYPE_VARIANT, "as");
+    if (int ret = sd_bus_message_open_container(msgRef(), SD_BUS_TYPE_VARIANT, "as"); ret < 0)
+        return ret;
+
+    return sd_bus_message_open_container(msgRef(), SD_BUS_TYPE_ARRAY, "s");
 }
 
 int ASValue::closeVariant() noexcept
 {
+    if (int ret = sd_bus_message_close_container(msgRef()); ret < 0)
+        return ret;
+
     return sd_bus_message_close_container(msgRef());
 }
 
 int ASValue::appendValue(std::string &&value) noexcept
 {
-    std::string envs{std::move(value)};
-    std::istringstream stream{envs};
-    int ret{0};
-
-    if (ret = sd_bus_message_open_container(msgRef(), SD_BUS_TYPE_ARRAY, "s"); ret < 0) {
-        return ret;
-    }
-
-    for (std::string line; std::getline(stream, line, ';');) {
-        if (ret = sd_bus_message_append(msgRef(), "s", line.data()); ret < 0) {
-            return ret;
-        }
-    }
-
-    return sd_bus_message_close_container(msgRef());
+    return sd_bus_message_append(msgRef(), "s", value.data());
 }
