@@ -4,23 +4,26 @@
 
 #include "dbus/applicationservice.h"
 #include "APPobjectmanager1adaptor.h"
+#include "applicationadaptor.h"
 #include "applicationchecker.h"
 #include "applicationmanagerstorage.h"
+#include "config.h"
 #include "constant.h"
-#include "global.h"
-#include "iniParser.h"
-#include "propertiesForwarder.h"
 #include "dbus/instanceadaptor.h"
-#include "launchoptions.h"
 #include "desktopentry.h"
 #include "desktopfileparser.h"
-#include <QUuid>
-#include <QStringList>
+#include "global.h"
+#include "iniParser.h"
+#include "launchoptions.h"
+#include "propertiesForwarder.h"
+#include <DConfig>
 #include <QList>
-#include <QUrl>
-#include <QRegularExpression>
 #include <QProcess>
+#include <QRegularExpression>
 #include <QStandardPaths>
+#include <QStringList>
+#include <QUrl>
+#include <QUuid>
 #include <algorithm>
 #include <new>
 #include <qcontainerfwd.h>
@@ -32,60 +35,100 @@
 #include <utility>
 #include <wordexp.h>
 
-double getScaleFactor() noexcept
+using namespace Qt::Literals::StringLiterals;
+
+static inline void appendEnvs(const QVariant &var, QStringList &envs)
 {
-    auto sessionBus = QDBusConnection::sessionBus();
-    QDBusMessage reply1 = sessionBus.call(QDBusMessage::createMethodCall(
-        "org.deepin.dde.XSettings1", "/org/deepin/dde/XSettings1", "org.deepin.dde.XSettings1", "GetScaleFactor"));
-
-    if (reply1.type() != QDBusMessage::ReplyMessage) {
-        qWarning() << "call GetScaleFactor Failed:" << reply1.errorMessage();
-        return 1.0;
+    if (var.canConvert<QStringList>()) {
+        envs.append(var.value<QStringList>());
+    } else if (var.canConvert<QString>()) {
+        envs.append(var.value<QString>().split(";", Qt::SkipEmptyParts));
     }
-
-    QDBusReply<double> ret1(reply1);
-    double scale = ret1.isValid() ? ret1.value() : 1.0;
-    scale = scale > 0 ? scale : 1;
-    return scale;
 }
 
 void ApplicationService::appendExtraEnvironments(QVariantMap &runtimeOptions) const noexcept
 {
-    QString oldEnv;
-    // scale factor
-    static QStringList scaleEnvs{"DEEPIN_WINE_SCALE=%1;", "QT_SCALE_FACTOR=%1;"};
-    auto factor = scaleFactor();
+    DCORE_USE_NAMESPACE
+    QStringList envs, unsetEnvs;
+    const QString &env = environ();
+    if (!env.isEmpty())
+        envs.append(env);
+
     if (auto it = runtimeOptions.find("env"); it != runtimeOptions.cend()) {
-        oldEnv = it->value<QString>();
+        appendEnvs(*it, envs);
     }
 
-    for (const auto &env : scaleEnvs) {
-        oldEnv.append(env.arg(factor));
+    if (auto it = runtimeOptions.find("unsetEnv"); it != runtimeOptions.cend()) {
+        appendEnvs(*it, unsetEnvs);
     }
 
-    // FIX #7528 GTK app only scale the UI, not scaling of text.
-    // GDK_SCALE Must be set to an integer, see https://docs.gtk.org/gtk3/x11.html
-    // 1.25 ==> 1 , 1.75 ==>2
-    int scale = qRound(factor);
-    oldEnv.append(QString("GDK_SCALE=%1;").arg(scale));
-    if (scale > 0) { // avoid division by 0
-        oldEnv.append(QString("GDK_DPI_SCALE=%1;").arg(qreal(1.0 / scale)));
+    std::unique_ptr<DConfig> config(DConfig::create(ApplicationServiceID,
+                                                    ApplicationManagerConfig,
+                                                    QString("/%1").arg((id()))));  // $appid as subpath
+    if (config->isValid()) {
+        const QStringList &extraEnvs = config->value(AppExtraEnvironments).toStringList();
+        if (!extraEnvs.isEmpty())
+            envs.append(extraEnvs);
+
+        const QStringList &envsBlacklist = config->value(AppEnvironmentsBlacklist).toStringList();
+        if (!envsBlacklist.isEmpty())
+            unsetEnvs.append(envsBlacklist);
     }
 
-    // GIO
-    oldEnv.append(QString{"GIO_LAUNCHED_DESKTOP_FILE=%1;"}.arg(m_desktopSource.sourcePath()));
+    // it's useful for App to get itself AppId.
+    envs.append(QString{"DSG_APP_ID=%1"}.arg(id()));
 
-    // set for dxcb platform plugin, which should use scale factor directly instead of processing double times.
-    oldEnv.append(QString{"D_DXCB_FORCE_OVERRIDE_HIDPI=1"});
+    runtimeOptions.insert("env", envs);
+    runtimeOptions.insert("unsetEnv", unsetEnvs);
+}
 
-    runtimeOptions.insert("env", oldEnv);
+void ApplicationService::processCompatibility(const QString &action, QVariantMap &options, QString &execStr)
+{
+    QString tempExecStr = execStr;
+    auto compatibilityManager = parent()->getCompatibilityManager();
+
+    auto getExec = [action, compatibilityManager](QString desktopID) -> QString {
+        std::optional<QString> execValue;
+        if (!action.isEmpty()) {
+            const auto &actionHeader = QString{"%1%2"}.arg(DesktopFileActionKey, action);
+            execValue = compatibilityManager->getExec(desktopID, actionHeader);
+        } else {
+            execValue = compatibilityManager->getExec(desktopID, DesktopFileEntryKey);
+        }
+
+        if (!execValue.has_value() || execValue->isEmpty()) {
+            return QString();
+        }
+        return execValue.value();
+    };
+
+    auto addEnv = [this, &options, compatibilityManager]() {
+        auto envValue = compatibilityManager->getEnv(m_desktopSource.desktopId(), DesktopFileEntryKey);
+        if (!envValue.isEmpty()) {
+            if (auto it = options.find("env"); it != options.cend()) {
+                auto value = it->toStringList();
+                value.append(envValue);
+                options.insert("env", value);
+            } else {
+                options.insert("env", envValue);
+            }
+        }
+    };
+
+    auto exec = getExec(m_desktopSource.desktopId());
+    if (!exec.isEmpty()) {
+        execStr = exec;
+        addEnv();
+        qInfo() << "get compatibility : " << m_desktopSource.desktopId() << " Exec : " << execStr;
+    }
+
+    return;
 }
 
 ApplicationService::ApplicationService(DesktopFile source,
                                        ApplicationManager1Service *parent,
                                        std::weak_ptr<ApplicationManager1Storage> storage)
     : QObject(parent)
-    , m_scaleFactor(getScaleFactor())
     , m_storage(std::move(storage))
     , m_desktopSource(std::move(source))
 {
@@ -139,30 +182,9 @@ ApplicationService::ApplicationService(DesktopFile source,
         m_launchedTimes = value.toLongLong();
     }
 
-    value = storagePtr->readApplicationValue(appId, ApplicationPropertiesGroup, ScaleFactor);
+    value = storagePtr->readApplicationValue(appId, ApplicationPropertiesGroup, ::Environ);
     if (!value.isNull()) {
-        bool ok{false};
-        auto tmp = value.toDouble(&ok);
-        if (ok) {
-            m_scaleFactor = tmp;
-            m_customScale = true;
-        }
-    }
-
-    if (!QDBusConnection::sessionBus().connect("org.deepin.dde.XSettings1",
-                                               "/org/deepin/dde/XSettings1",
-                                               "org.deepin.dde.XSettings1",
-                                               "SetScaleFactorDone",
-                                               this,
-                                               SLOT(onGlobalScaleFactorChanged()))) {
-        qWarning() << "connect to org.deepin.dde.XSettings1 failed, scaleFactor is invalid.";
-    }
-}
-
-void ApplicationService::onGlobalScaleFactorChanged() noexcept
-{
-    if (!m_customScale) {
-        m_scaleFactor = getScaleFactor();
+        m_environ = value.toString();
     }
 }
 
@@ -196,7 +218,7 @@ QSharedPointer<ApplicationService> ApplicationService::createApplicationService(
     auto error = entry->parse(sourceStream);
 
     if (error != ParserError::NoError) {
-        qWarning() << "parse failed:" << error << app->desktopFileSource().sourcePath();
+        qDebug() << "parse failed:" << error << app->desktopFileSource().sourcePath();
         return nullptr;
     }
 
@@ -274,8 +296,8 @@ ApplicationService::Launch(const QString &action, const QStringList &fields, con
         execStr = toString(actionExec.value());
         if (execStr.isEmpty()) {
             qWarning() << "exec value to string failed, try default action.";  // we need this log.
-            break;
         }
+
         break;
     }
 
@@ -284,9 +306,7 @@ ApplicationService::Launch(const QString &action, const QStringList &fields, con
         if (!Actions) {
             QString msg{"application can't be executed."};
             qWarning() << msg;
-            if (calledFromDBus()) {
-                sendErrorReply(QDBusError::Failed, msg);
-            }
+            safe_sendErrorReply(QDBusError::Failed, msg);
             return {};
         }
 
@@ -294,9 +314,7 @@ ApplicationService::Launch(const QString &action, const QStringList &fields, con
         if (execStr.isEmpty()) {
             QString msg{"maybe entry actions's format is invalid, abort launch."};
             qWarning() << msg;
-            if (calledFromDBus()) {
-                sendErrorReply(QDBusError::Failed, msg);
-            }
+            safe_sendErrorReply(QDBusError::Failed, msg);
             return {};
         }
     }
@@ -307,75 +325,109 @@ ApplicationService::Launch(const QString &action, const QStringList &fields, con
     }
     optionsMap.insert("_builtIn_searchExec", parent()->systemdPathEnv());
 
+    processCompatibility(action, optionsMap, execStr);
+    unescapeEens(optionsMap);
+
     auto cmds = generateCommand(optionsMap);
     auto task = unescapeExec(execStr, fields);
     if (!task) {
-        if (calledFromDBus()) {
-            sendErrorReply(QDBusError::InternalError, "Invalid Command.");
-        }
+        safe_sendErrorReply(QDBusError::InternalError, "Invalid Command.");
         return {};
     }
 
-    auto [bin, execCmds, res] = std::move(task);
-    if (bin.isEmpty()) {
+    if (task.LaunchBin.isEmpty()) {
         qCritical() << "error command is detected, abort.";
-        if (calledFromDBus()) {
-            sendErrorReply(QDBusError::Failed);
-        }
+        safe_sendErrorReply(QDBusError::Failed);
         return {};
     }
 
     if (terminal()) {
         // don't change this sequence
-        execCmds.push_front("-C");           // means run a shellscript
-        execCmds.push_front("--keep-open");  // keep terminal open, prevent exit immediately
-        execCmds.push_front("deepin-terminal");
+        cmds.push_back("deepin-terminal");
+        cmds.push_back("--keep-open");  // keep terminal open, prevent exit immediately
+        cmds.push_back("-e");           // run all original execution commands in deepin-terminal
     }
-    cmds.append(std::move(execCmds));
 
     auto &jobManager = parent()->jobManager();
     return jobManager.addJob(
         m_applicationPath.path(),
-        [this, binary = std::move(bin), commands = std::move(cmds)](const QVariant &variantValue) -> QVariant {
-            auto resourceFile = variantValue.toString();
+        [this, task, cmds = std::move(cmds)](const QVariant &value) -> QVariant {  // do not change it to mutable lambda
             auto instanceRandomUUID = QUuid::createUuid().toString(QUuid::Id128);
             auto objectPath = m_applicationPath.path() + "/" + instanceRandomUUID;
-            auto newCommands = commands;
+            auto newCommands = cmds;
 
-            newCommands.push_front(QString{"--SourcePath=%1"}.arg(m_desktopSource.sourcePath()));
-            auto location = newCommands.indexOf(R"(%f)");
-            if (location != -1) {  // due to std::move, there only remove once
-                newCommands.remove(location);
-            }
-
-            if (resourceFile.isEmpty()) {
+            if (value.isNull()) {
+                newCommands.push_front(QString{"--SourcePath=%1"}.arg(m_desktopSource.sourcePath()));
                 newCommands.push_front(QString{R"(--unitName=app-DDE-%1@%2.service)"}.arg(
                     escapeApplicationId(this->id()), instanceRandomUUID));  // launcher should use this instanceId
+                newCommands.append(task.command);
+
                 QProcess process;
-                qDebug() << "run with commands:" << newCommands;
+                qDebug() << "launcher :" << m_launcher << "run with commands:" << newCommands;
                 process.start(m_launcher, newCommands);
                 process.waitForFinished();
                 if (auto code = process.exitCode(); code != 0) {
                     qWarning() << "Launch Application Failed";
                     return QDBusError::Failed;
                 }
+
                 return objectPath;
             }
 
-            auto url = QUrl::fromUserInput(resourceFile);
-            if (!url.isValid()) {  // if url is invalid, passing to launcher directly
-                auto scheme = url.scheme();
-                if (!scheme.isEmpty()) {
-                    // TODO: resourceFile = processRemoteFile(resourceFile);
+            if (task.argNum != -1) {
+                if (task.argNum >= newCommands.size()) {
+                    qCritical() << "task.argNum >= task.command.size()";
+                    return QDBusError::Failed;
                 }
+
+                QStringList rawRes;
+                if (value.canConvert<QStringList>()) {  // from %F, %U
+                    rawRes = value.toStringList();
+                } else if (value.canConvert<QString>()) {  // from %f, %u
+                    rawRes.append(value.toString());
+                } else {
+                    qWarning() << "value type mismatch:" << value;
+                    return QDBusError::Failed;
+                }
+
+                if (task.local) {
+                    std::for_each(rawRes.begin(), rawRes.end(), [](QString &str) {
+                        auto url = QUrl::fromUserInput(str);
+
+                        if (url.isLocalFile()) {
+                            str = url.toLocalFile();
+                            return;
+                        }
+
+                        // TODO: processing remote files
+                        // str = downloadToLocal(url).toLocalFile();
+                    });
+                }
+
+                auto newCmds = task.command;
+                if (task.fieldLocation == -1) {  // single field, e.g. "demo %U"
+                    auto it = newCmds.begin() + task.argNum + 1;
+                    std::for_each(
+                        rawRes.rbegin(), rawRes.rend(), [&newCmds, &it](QString &str) { it = newCmds.insert(it, str); });
+                } else {
+                    auto arg = newCmds.begin() + task.argNum + 1;
+                    arg->insert(task.fieldLocation, rawRes.takeFirst());
+                    ++arg;
+
+                    // expand the rest of res
+                    std::for_each(
+                        rawRes.rbegin(), rawRes.rend(), [&newCmds, &arg](QString &str) { arg = newCmds.insert(arg, str); });
+                }
+
+                newCommands.append(std::move(newCmds));
             }
 
-            // NOTE: resourceFile must be available in the following contexts
-            newCommands.insert(location, resourceFile);
+            newCommands.push_front(QString{"--SourcePath=%1"}.arg(m_desktopSource.sourcePath()));
+            newCommands.push_front(
+                QString{R"(--unitName=app-DDE-%1@%2.service)"}.arg(escapeApplicationId(this->id()), instanceRandomUUID));
 
-            newCommands.push_front(QString{R"(--unitName=DDE-%1@%2.service)"}.arg(this->id(), instanceRandomUUID));
             QProcess process;
-            qDebug() << "run with commands:" << newCommands;
+            qDebug().noquote() << "launcher :" << m_launcher << "run with commands:" << newCommands;
             process.start(getApplicationLauncherBinary(), newCommands);
             process.waitForFinished();
             auto exitCode = process.exitCode();
@@ -383,9 +435,10 @@ ApplicationService::Launch(const QString &action, const QStringList &fields, con
                 qWarning() << "Launch Application Failed";
                 return QDBusError::Failed;
             }
+
             return objectPath;
         },
-        std::move(res));
+        std::move(task.Resources));
 }
 
 bool ApplicationService::SendToDesktop() const noexcept
@@ -404,7 +457,7 @@ bool ApplicationService::SendToDesktop() const noexcept
     auto success = m_desktopSource.sourceFileRef().link(desktopFile);
     if (!success) {
         qDebug() << "create link failed:" << m_desktopSource.sourceFileRef().errorString() << "path:" << desktopFile;
-        sendErrorReply(QDBusError::ErrorType::Failed, m_desktopSource.sourceFileRef().errorString());
+        safe_sendErrorReply(QDBusError::ErrorType::Failed, m_desktopSource.sourceFileRef().errorString());
     }
 
     return success;
@@ -427,7 +480,7 @@ bool ApplicationService::RemoveFromDesktop() const noexcept
 
     if (!success) {
         qDebug() << "remove desktop file failed:" << desktopFile.errorString();
-        sendErrorReply(QDBusError::ErrorType::Failed, desktopFile.errorString());
+        safe_sendErrorReply(QDBusError::ErrorType::Failed, desktopFile.errorString());
     }
 
     return success;
@@ -582,6 +635,38 @@ QString ApplicationService::X_Deepin_Vendor() const noexcept
     return findEntryValue(DesktopFileEntryKey, "X-Deepin-Vendor", EntryValueType::String).toString();
 }
 
+QString ApplicationService::X_Deepin_CreateBy() const noexcept
+{
+    return findEntryValue(DesktopFileEntryKey, "X-Deepin-CreateBy", EntryValueType::String).toString();
+}
+
+QStringMap ApplicationService::execs() const noexcept
+{
+    QStringMap ret;
+
+    auto mainExec = m_entry->value(DesktopFileEntryKey, "Exec");
+    if (mainExec.has_value()) {
+        ret.insert(DesktopFileEntryKey, mainExec->value<QString>());
+    }
+
+    auto actionList = actions();
+    for (const auto &action : actionList) {
+        auto actionKey = QString{action}.prepend(DesktopFileActionKey);
+        auto value = m_entry->value(actionKey, "Exec");
+        if (!value.has_value()) {
+            continue;
+        }
+        ret.insert(actionKey, value->value<QString>());
+    }
+
+    return ret;
+}
+
+QString ApplicationService::X_CreatedBy() const noexcept
+{
+    return findEntryValue(DesktopFileEntryKey, "X-Created-By", EntryValueType::String).toString();
+}
+
 bool ApplicationService::terminal() const noexcept
 {
     auto val = findEntryValue(DesktopFileEntryKey, "Terminal", EntryValueType::String);
@@ -591,6 +676,12 @@ bool ApplicationService::terminal() const noexcept
     return false;
 }
 
+QString ApplicationService::startupWMClass() const noexcept
+{
+    auto value = findEntryValue(DesktopFileEntryKey, "StartupWMClass", EntryValueType::String);
+    return value.isNull() ? QString{} : value.toString();
+}
+
 qint64 ApplicationService::installedTime() const noexcept
 {
     return m_installedTime;
@@ -606,43 +697,38 @@ qint64 ApplicationService::launchedTimes() const noexcept
     return m_launchedTimes;
 }
 
-double ApplicationService::scaleFactor() const noexcept
+QString ApplicationService::environ() const noexcept
 {
-    return m_scaleFactor;
+    return m_environ;
 }
 
-void ApplicationService::setScaleFactor(double value) noexcept
+void ApplicationService::setEnviron(const QString &value) noexcept
 {
+    if (environ().trimmed() == value.trimmed()) {
+        return;
+    }
+
     auto storagePtr = m_storage.lock();
     if (!storagePtr) {
         qCritical() << "broken storage.";
-        sendErrorReply(QDBusError::InternalError);
+        safe_sendErrorReply(QDBusError::InternalError);
         return;
     }
 
     auto appId = id();
-    if (value == 0) {
-        m_customScale = false;
-        m_scaleFactor = getScaleFactor();
-        if (!storagePtr->deleteApplicationValue(appId, ApplicationPropertiesGroup, ScaleFactor)) {
-            qCritical() << "failed to delete app's property:" << appId;
-        }
-        return;
-    }
-
-    if (m_customScale) {
-        if (!storagePtr->updateApplicationValue(appId, ApplicationPropertiesGroup, ScaleFactor, value)) {
-            sendErrorReply(QDBusError::Failed, "update scaleFactor failed.");
+    if (!storagePtr->readApplicationValue(appId, ApplicationPropertiesGroup, Environ).isNull()) {
+        if (!storagePtr->updateApplicationValue(appId, ApplicationPropertiesGroup, Environ, value)) {
+            safe_sendErrorReply(QDBusError::Failed, "update environ failed.");
             return;
         }
     } else {
-        if (!storagePtr->createApplicationValue(appId, ApplicationPropertiesGroup, ScaleFactor, value)) {
-            sendErrorReply(QDBusError::Failed, "set scaleFactor failed.");
+        if (!storagePtr->createApplicationValue(appId, ApplicationPropertiesGroup, Environ, value)) {
+            safe_sendErrorReply(QDBusError::Failed, "set environ failed.");
         }
     }
 
-    m_scaleFactor = value;
-    emit scaleFactorChanged();
+    m_environ = value;
+    emit environChanged();
 }
 
 bool ApplicationService::autostartCheck(const QString &filePath) const noexcept
@@ -666,6 +752,12 @@ bool ApplicationService::autostartCheck(const QString &filePath) const noexcept
         }
     }
 
+    QString source = s.value(DesktopFileEntryKey, X_Deepin_GenerateSource).value_or(DesktopEntry::Value{}).toString();
+    // file has been removed
+    if (source != m_autostartSource.m_filePath && filePath != m_autostartSource.m_filePath) {
+        return false;
+    }
+
     auto hiddenVal = s.value(DesktopFileEntryKey, DesktopEntryHidden);
     if (!hiddenVal) {
         qDebug() << "no hidden in autostart desktop";
@@ -699,11 +791,6 @@ bool ApplicationService::isAutoStart() const noexcept
         {"*.desktop"},
         QDir::Name | QDir::DirsLast);
 
-    // file has been removed
-    if (destDesktopFile != m_autostartSource.m_filePath) {
-        return false;
-    }
-
     return autostartCheck(destDesktopFile);
 }
 
@@ -713,22 +800,34 @@ void ApplicationService::setAutoStart(bool autostart) noexcept
         return;
     }
 
-    auto fileName = QDir{getAutoStartDirs().first()}.filePath(m_desktopSource.desktopId() + ".desktop");
+    QDir startDir(getAutoStartDirs().first());
+    if (!startDir.exists() && !startDir.mkpath(startDir.path())) {
+        qWarning() << "mkpath " << startDir.path() << "failed";
+        safe_sendErrorReply(QDBusError::InternalError);
+        return;
+    }
+
+    auto fileName = startDir.filePath(m_desktopSource.desktopId() + ".desktop");
     QFile autostartFile{fileName};
     if (!autostartFile.open(QFile::WriteOnly | QFile::Text | QFile::Truncate)) {
         qWarning() << "open file" << fileName << "failed:" << autostartFile.error();
-        sendErrorReply(QDBusError::Failed);
+        safe_sendErrorReply(QDBusError::Failed);
         return;
     }
 
     DesktopEntry newEntry;
+    QString originalSource;
     if (!m_autostartSource.m_entry.data().isEmpty()) {
         newEntry = m_autostartSource.m_entry;
+        originalSource = newEntry.value(DesktopFileEntryKey, X_Deepin_GenerateSource)
+                             .value_or(DesktopEntry::Value{m_autostartSource.m_filePath})
+                             .toString();
     } else {
         newEntry = *m_entry;
+        originalSource = m_desktopSource.sourcePath();
     }
 
-    newEntry.insert(DesktopFileEntryKey, X_Deepin_GenerateSource, m_autostartSource.m_filePath);
+    newEntry.insert(DesktopFileEntryKey, X_Deepin_GenerateSource, originalSource);
     newEntry.insert(DesktopFileEntryKey, DesktopEntryHidden, !autostart);
 
     setAutostartSource({fileName, newEntry});
@@ -738,7 +837,7 @@ void ApplicationService::setAutoStart(bool autostart) noexcept
 
     if (writeBytes != hideAutostart.size() or !autostartFile.flush()) {
         qWarning() << "incomplete write:" << autostartFile.error();
-        sendErrorReply(QDBusError::Failed, "set failed: filesystem error.");
+        safe_sendErrorReply(QDBusError::Failed, "set failed: filesystem error.");
         return;
     }
 
@@ -754,6 +853,11 @@ QStringList ApplicationService::mimeTypes() const noexcept
         return desktopFilePath.startsWith(info.directory());
     });
 
+    if (cache == cacheList.cend()) {
+        qWarning() << "error occurred when get mimeTypes for" << desktopFilePath;
+        return ret;
+    }
+
     const auto &info = cache->cacheInfo();
     if (info) {
         ret.append(info->queryTypes(id()));
@@ -798,7 +902,7 @@ void ApplicationService::setMimeTypes(const QStringList &value) noexcept
     auto &infos = parent()->mimeManager().infos();
     auto userInfo = std::find_if(infos.begin(), infos.end(), [](const MimeInfo &info) { return info.directory() == userDir; });
     if (userInfo == infos.cend()) {
-        sendErrorReply(QDBusError::Failed, "user-specific config file doesn't exists.");
+        safe_sendErrorReply(QDBusError::Failed, "user-specific config file doesn't exists.");
         return;
     }
 
@@ -907,8 +1011,12 @@ void ApplicationService::resetEntry(DesktopEntry *newEntry) noexcept
     emit categoriesChanged();
     emit MimeTypesChanged();
     emit terminalChanged();
-    emit scaleFactorChanged();
+    emit environChanged();
     emit launchedTimesChanged();
+    emit startupWMClassChanged();
+    emit xDeepinCreatedByChanged();
+    emit execsChanged();
+    emit xCreatedByChanged();
 }
 
 std::optional<QStringList> ApplicationService::unescapeExecArgs(const QString &str) noexcept
@@ -965,159 +1073,203 @@ std::optional<QStringList> ApplicationService::unescapeExecArgs(const QString &s
     return execList;
 }
 
-LaunchTask ApplicationService::unescapeExec(const QString &str, const QStringList &fields) noexcept
+LaunchTask ApplicationService::unescapeExec(const QString &str, QStringList fields) noexcept
 {
     LaunchTask task;
-    auto opt = unescapeExecArgs(str);
+    auto args = unescapeExecArgs(str);
 
-    if (!opt.has_value()) {
+    if (!args) {
         qWarning() << "unescapeExecArgs failed.";
         return {};
     }
 
-    auto execList = std::move(opt).value();
-    if (execList.isEmpty()) {
+    if (args->isEmpty()) {
         qWarning() << "exec format is invalid.";
         return {};
     }
 
-    task.LaunchBin = execList.first();
-    QRegularExpression re{"%[fFuUickdDnNvm]"};
-    auto matcher = re.match(str);
-    if (!matcher.hasMatch()) {
-        task.command.append(std::move(execList));
-        task.Resources.emplace_back(QString{""});  // mapReduce should run once at least
-        return task;
-    }
+    task.LaunchBin = args->first();
+    const QChar percentage{'%'};
+    bool exclusiveField{false};
 
-    auto list = matcher.capturedTexts();
-    if (list.count() != 1) {
-        qWarning() << "invalid exec format, all filed code will be ignored.";
-        for (const auto &code : list) {
-            execList.removeOne(code);
-        }
-        task.command.append(std::move(execList));
-        return task;
-    }
+    for (auto arg = args->begin(); arg != args->end(); ++arg) {
+        QString newArg;
 
-    auto filesCode = list.first().back().toLatin1();
-    auto codeStr = QString(R"(%%1)").arg(filesCode);
-    auto location = execList.indexOf(codeStr);
-
-    switch (filesCode) {
-    case 'f': {  // Defer to async job
-        task.command.append(std::move(execList));
-        for (const auto &field : fields) {
-            task.Resources.emplace_back(field);
-        }
-    } break;
-    case 'u': {
-        execList.removeAt(location);
-        if (fields.empty()) {
-            task.command.append(execList);
-            break;
-        }
-        if (fields.count() > 1) {
-            qDebug() << R"(fields count is greater than one, %u will only take first element.)";
-        }
-        execList.insert(location, fields.first());
-        task.command.append(execList);
-    } break;
-    case 'F': {
-        execList.remove(location);
-        auto it = execList.begin() + location;
-        for (const auto &field : fields) {
-            auto tmp = QUrl{field};
-            if (auto scheme = tmp.scheme(); scheme.startsWith("file") or scheme.isEmpty()) {
-                it = execList.insert(it, tmp.toLocalFile());
-            } else {
-                qWarning() << "shouldn't replace %F with an URL:" << field;
-                it = execList.insert(it, field);
+        for (const auto *it = arg->cbegin(); it != arg->cend();) {
+            if (*it != percentage) {
+                newArg.append(*(it++));
+                continue;
             }
-            ++it;
-        }
-        task.command.append(std::move(execList));
-    } break;
-    case 'U': {
-        execList.removeAt(location);
-        auto it = execList.begin() + location;
-        for (const auto &field : fields) {
-            it = execList.insert(it, field);
-            ++it;
-        }
-        task.command.append(std::move(execList));
-    } break;
-    case 'i': {
-        execList.removeAt(location);
-        auto val = m_entry->value(DesktopFileEntryKey, "Icon");
-        if (!val) {
-            qDebug() << R"(Application Icons can't be found. %i will be ignored.)";
-            task.command.append(std::move(execList));
-            return task;
+
+            const auto *code = it + 1;
+            if (code == arg->cend()) {
+                qWarning() << R"(content of exec is invalid, a unterminated % is detected.)";
+                return {};
+            }
+
+            if (*code == percentage) {
+                newArg.append(percentage);
+                it += 2;
+                continue;
+            }
+
+            switch (auto c = code->toLatin1(); c) {
+            case 'f':
+                [[fallthrough]];
+            case 'F': {  // Defer to async job
+                if (exclusiveField) {
+                    qDebug() << QString{"exclusive field is detected again, %%1 will be ignored."}.arg(c);
+                    break;
+                }
+                exclusiveField = true;
+
+                if (fields.empty()) {
+                    qDebug() << QString{"fields is empty, %%1 will be ignored."}.arg(c);
+                    break;
+                }
+
+                if (c == 'F') {
+                    task.Resources.emplace_back(fields);
+                } else {
+                    std::for_each(
+                        fields.begin(), fields.end(), [&task](QString &str) { task.Resources.emplace_back(std::move(str)); });
+                }
+
+                task.argNum = std::distance(args->begin(), arg) - 1;
+                task.fieldLocation = std::distance(arg->cbegin(), it) - 1;
+                task.local = true;
+            } break;
+            case 'u':
+            case 'U': {
+                if (exclusiveField) {
+                    qDebug() << QString{"exclusive field is detected again, %%1 will be ignored."}.arg(c);
+                    break;
+                }
+                exclusiveField = true;
+
+                if (fields.empty()) {
+                    qDebug() << QString{"fields is empty, %%1 will be ignored."}.arg(c);
+                    break;
+                }
+
+                // respect the original url, pass it to exec directly
+                if (c == 'U') {
+                    task.Resources.emplace_back(std::move(fields));
+                } else {
+                    std::for_each(
+                        fields.begin(), fields.end(), [&task](QString &url) { task.Resources.emplace_back(std::move(url)); });
+                }
+
+                task.argNum = std::distance(args->begin(), arg) - 1;
+                task.fieldLocation = std::distance(arg->cbegin(), it) - 1;
+            } break;
+            case 'i': {
+                auto val = m_entry->value(DesktopFileEntryKey, "Icon");
+                if (!val) {
+                    qDebug() << R"(Application Icons can't be found. %i will be ignored.)";
+                    break;
+                }
+
+                auto iconStr = toIconString(val.value());
+                if (iconStr.isEmpty()) {
+                    qDebug() << R"(Icons Convert to string failed. %i will be ignored.)";
+                    break;
+                }
+
+                // split at the end of loop
+                newArg.append(QString{"--icon %1"}.arg(iconStr));
+            } break;
+            case 'c': {
+                auto val = m_entry->value(DesktopFileEntryKey, "Name");
+                if (!val) {
+                    qDebug() << R"(Application Name can't be found. %c will be ignored.)";
+                    break;
+                }
+
+                const auto &rawValue = val.value();
+                if (!rawValue.canConvert<QStringMap>()) {
+                    qDebug() << "Name's underlying type mismatch:" << "QStringMap" << rawValue.metaType().name();
+                    break;
+                }
+
+                auto NameStr = toLocaleString(rawValue.value<QStringMap>(), getUserLocale());
+                if (NameStr.isEmpty()) {
+                    qDebug() << R"(Name Convert to locale string failed. %c will be ignored.)";
+                    break;
+                }
+
+                newArg.append(NameStr);
+            } break;
+            case 'k': {  // ignore all desktop file location for now.
+                newArg.append(m_desktopSource.sourcePath());
+            } break;
+            case 'd':
+            case 'D':
+            case 'n':
+            case 'N':
+            case 'v':
+                [[fallthrough]];  // Deprecated field codes should be removed from the command line and ignored.
+            case 'm': {
+                qDebug() << "field code" << *code << "has been deprecated.";
+            } break;
+            default: {
+                qDebug() << "unknown field code:" << *code << ", ignore it.";
+            }
+            }
+
+            it += 2;  // skip filed code
         }
 
-        auto iconStr = toIconString(val.value());
-        if (iconStr.isEmpty()) {
-            qDebug() << R"(Icons Convert to string failed. %i will be ignored.)";
-            task.command.append(std::move(execList));
-            return task;
-        }
-        auto it = execList.insert(location, iconStr);
-        execList.insert(it, "--icon");
-        task.command.append(std::move(execList));
-    } break;
-    case 'c': {
-        execList.removeAt(location);
-        auto val = m_entry->value(DesktopFileEntryKey, "Name");
-        if (!val) {
-            qDebug() << R"(Application Name can't be found. %c will be ignored.)";
-            task.command.append(std::move(execList));
-            return task;
+        // unescapeExecArgs()函数中使用了遵循POSIX标准的wordexp，可以正确解析被引号包裹的整体，解析成参数列表，
+        // 被分割的都是一个个整体，若分割后的参数在这里还能再按空格分割，
+        // 则说明这个参数被包裹在一个引号中作为了一个整体，此时我们不能再分割这个参数
+        const bool noSplit = (*arg).split(' ').size() > 1;
+        QStringList newArgList;
+        if (noSplit) {
+            newArgList = {newArg};
+        } else {
+            newArgList = newArg.split(' ', Qt::SkipEmptyParts);
         }
 
-        const auto &rawValue = val.value();
-        if (!rawValue.canConvert<QStringMap>()) {
-            qDebug() << "Name's underlying type mismatch:"
-                     << "QStringMap" << rawValue.metaType().name();
-            task.command.append(std::move(execList));
-            return task;
+        if (!newArgList.isEmpty()) {
+            task.command.append(std::move(newArgList));
         }
-
-        auto NameStr = toLocaleString(rawValue.value<QStringMap>(), getUserLocale());
-        if (NameStr.isEmpty()) {
-            qDebug() << R"(Name Convert to locale string failed. %c will be ignored.)";
-            task.command.append(std::move(execList));
-            return task;
-        }
-        execList.insert(location, NameStr);
-        task.command.append(std::move(execList));
-    } break;
-    case 'k': {  // ignore all desktop file location for now.
-        execList.removeAt(location);
-        task.command.append(std::move(execList));
-    } break;
-    case 'd':
-    case 'D':
-    case 'n':
-    case 'N':
-    case 'v':
-        [[fallthrough]];  // Deprecated field codes should be removed from the command line and ignored.
-    case 'm': {
-        execList.removeAt(location);
-        task.command.append(std::move(execList));
-    } break;
-    default: {
-        qDebug() << "unrecognized file code.";
-    }
     }
 
     if (task.Resources.isEmpty()) {
-        task.Resources.emplace_back(QString{""});  // mapReduce should run once at least
+        task.Resources.emplace_back(QVariant{});  // mapReduce should run once at least
     }
 
+    qInfo() << "after unescape exec:" << task.LaunchBin << task.command << task.Resources;
     return task;
 }
 
+void ApplicationService::unescapeEens(QVariantMap &options) noexcept
+{
+    if (options.find("env") == options.end()) {
+        return;
+    }
+    QStringList result;
+    auto envs = options["env"];
+    for (const QString &var : envs.toStringList()) {
+        if (var.startsWith(u"DSG_APP_ID="_s)) {
+            result << var;
+            continue;
+        }
+        wordexp_t p;
+        if (wordexp(var.toStdString().c_str(), &p, 0) == 0) {
+            for (size_t i = 0; i < p.we_wordc; i++) {
+                result << QString::fromLocal8Bit(p.we_wordv[i]);  // 将结果转换为QString
+            }
+            wordfree(&p);
+        } else {
+            return;
+        }
+    }
+
+    options.insert("env", result);
+}
+
 QVariant ApplicationService::findEntryValue(const QString &group,
                                             const QString &valueKey,
                                             EntryValueType type,
@@ -1206,4 +1358,5 @@ void ApplicationService::updateAfterLaunch(bool isLaunch) noexcept
 void ApplicationService::setAutostartSource(AutostartSource &&source) noexcept
 {
     m_autostartSource = std::move(source);
+    emit autostartChanged();
 }
