@@ -24,6 +24,7 @@
 #include <QLoggingCategory>
 #include <sys/stat.h>
 #include <sys/syscall.h>
+#include <signal.h>
 #include "constant.h"
 #include "config.h"
 
@@ -395,6 +396,11 @@ inline QString getXDGDataHome()
     return XDGDataHome;
 }
 
+inline QStringList getXDGSystemDirs()
+{
+    return QStringList{"/etc/xdg/autostart"};
+}
+
 inline QStringList getXDGDataDirs()
 {
     auto XDGDataDirs = QString::fromLocal8Bit(qgetenv("XDG_DATA_DIRS")).split(':', Qt::SkipEmptyParts);
@@ -481,11 +487,6 @@ inline QString getCurrentDesktop()
     return desktops.first();
 }
 
-inline bool isApplication(const QDBusObjectPath &path)
-{
-    return path.path().split('/').last().startsWith("app");
-}
-
 struct unitInfo
 {
     QString applicationID;
@@ -503,8 +504,10 @@ inline unitInfo processUnitName(const QString &unitName)
 
     decltype(auto) appPrefix = u8"app-";
     if (!unitName.startsWith(appPrefix)) {
-        qDebug() << "this unit " << unitName << "isn't an app unit.";
-        return {};
+        // If not started by application manager, just remove suffix and take name as app id.
+        auto lastDotIndex = unitName.lastIndexOf('.');
+        applicationId = unitName.sliced(0, lastDotIndex);
+        return {unescapeApplicationId(applicationId), std::move(launcher), std::move(instanceId)};
     }
 
     auto unit = unitName.sliced(sizeof(appPrefix) - 1);
@@ -540,22 +543,24 @@ inline unitInfo processUnitName(const QString &unitName)
         return {};
     }
 
-    if (instanceId.isEmpty()) {
-        instanceId = QUuid::createUuid().toString(QUuid::Id128);
-    }
-
     return {unescapeApplicationId(applicationId), std::move(launcher), std::move(instanceId)};
 }
 
-template <typename T>
-ObjectMap dumpDBusObject(const QMap<QDBusObjectPath, QSharedPointer<T>> &map)
+template <typename Key, typename Value>
+ObjectMap dumpDBusObject(const QHash<Key, QSharedPointer<Value>> &map)
 {
+    static_assert(std::is_base_of_v<QObject, Value>, "dumpDBusObject only support which derived by QObject class");
+    static_assert(std::is_same_v<Key, QString> || std::is_same_v<Key, QDBusObjectPath>,
+                  "dumpDBusObject only support QString/QDBusObject as key type");
     ObjectMap objs;
 
-    for (auto it = map.constKeyValueBegin(); it != map.constKeyValueEnd(); ++it) {
-        const auto &[key, value] = *it;
+    for (const auto &[key, value] : map.asKeyValueRange()) {
         auto interAndProps = getChildInterfacesAndPropertiesFromObject(value.data());
-        objs.insert(key, interAndProps);
+        if constexpr (std::is_same_v<Key, QString>) {
+            objs.insert(QDBusObjectPath{getObjectPathFromAppId(key)}, interAndProps);
+        } else if constexpr (std::is_same_v<Key, QDBusObjectPath>) {
+            objs.insert(key, interAndProps);
+        }
     }
 
     return objs;
@@ -658,4 +663,13 @@ inline int pidfd_open(pid_t pid, uint flags)
     return syscall(SYS_pidfd_open, pid, flags);
 }
 
+inline int pidfd_send_signal(int pidfd, int sig, siginfo_t *info, unsigned int flags)
+{
+    return syscall(SYS_pidfd_send_signal, pidfd, sig, info, flags);
+}
+
+#define safe_sendErrorReply                                                                                                      \
+    if (calledFromDBus())                                                                                                        \
+    sendErrorReply
+
 #endif
